CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
184 lines
6.6 KiB
C++
184 lines
6.6 KiB
C++
/*
|
|
* Copyright (c) Advanced Micro Devices, Inc., or its affiliates.
|
|
*
|
|
* SPDX-License-Identifier: MIT
|
|
*/
|
|
|
|
#include "memcpy_performance_common.hh"
|
|
|
|
/**
|
|
* @addtogroup memcpy memcpy
|
|
* @{
|
|
* @ingroup PerformanceTestMemory
|
|
*/
|
|
|
|
class Memcpy3DAsyncBenchmark : public Benchmark<Memcpy3DAsyncBenchmark> {
|
|
public:
|
|
void operator()(const hipPitchedPtr& dst_ptr, const hipPitchedPtr& src_ptr,
|
|
const hipExtent extent, hipMemcpyKind kind, const hipStream_t& stream) {
|
|
hipMemcpy3DParms params = CreateMemcpy3DParam(dst_ptr, make_hipPos(0, 0, 0), src_ptr,
|
|
make_hipPos(0, 0, 0), extent, kind);
|
|
TIMED_SECTION_STREAM(kTimerTypeEvent, stream) { HIP_CHECK(hipMemcpy3DAsync(¶ms, stream)); }
|
|
HIP_CHECK(hipStreamSynchronize(stream));
|
|
}
|
|
};
|
|
|
|
static void RunBenchmark(const hipExtent extent, hipMemcpyKind kind,
|
|
bool enable_peer_access = false) {
|
|
Memcpy3DAsyncBenchmark benchmark;
|
|
benchmark.AddSectionName("(" + std::to_string(extent.width) + ", " +
|
|
std::to_string(extent.height) + ", " + std::to_string(extent.depth) +
|
|
")");
|
|
|
|
const StreamGuard stream_guard(Streams::created);
|
|
const hipStream_t stream = stream_guard.stream();
|
|
|
|
if (kind == hipMemcpyDeviceToHost) {
|
|
LinearAllocGuard3D<int> device_allocation(extent);
|
|
LinearAllocGuard<int> host_allocation(
|
|
LinearAllocs::hipHostMalloc,
|
|
device_allocation.width() * device_allocation.height() * device_allocation.depth());
|
|
benchmark.Run(make_hipPitchedPtr(host_allocation.ptr(), device_allocation.width(),
|
|
device_allocation.width(), device_allocation.height()),
|
|
device_allocation.pitched_ptr(), device_allocation.extent(), kind, stream);
|
|
} else if (kind == hipMemcpyHostToDevice) {
|
|
LinearAllocGuard3D<int> device_allocation(extent);
|
|
LinearAllocGuard<int> host_allocation(
|
|
LinearAllocs::hipHostMalloc,
|
|
device_allocation.pitch() * device_allocation.height() * device_allocation.depth());
|
|
benchmark.Run(device_allocation.pitched_ptr(),
|
|
make_hipPitchedPtr(host_allocation.ptr(), device_allocation.pitch(),
|
|
device_allocation.width(), device_allocation.height()),
|
|
device_allocation.extent(), kind, stream);
|
|
} else if (kind == hipMemcpyHostToHost) {
|
|
LinearAllocGuard3D<int> device_allocation(extent);
|
|
LinearAllocGuard<int> src_allocation(LinearAllocs::hipHostMalloc,
|
|
extent.width * extent.height * extent.depth);
|
|
LinearAllocGuard<int> dst_allocation(LinearAllocs::hipHostMalloc,
|
|
extent.width * extent.height * extent.depth);
|
|
benchmark.Run(
|
|
make_hipPitchedPtr(dst_allocation.ptr(), extent.width, extent.width, extent.height),
|
|
make_hipPitchedPtr(src_allocation.ptr(), extent.width, extent.width, extent.height), extent,
|
|
kind, stream);
|
|
} else {
|
|
// hipMemcpyDeviceToDevice
|
|
int src_device = std::get<0>(GetDeviceIds(enable_peer_access));
|
|
int dst_device = std::get<1>(GetDeviceIds(enable_peer_access));
|
|
|
|
LinearAllocGuard3D<int> src_allocation(extent);
|
|
HIP_CHECK(hipSetDevice(dst_device));
|
|
LinearAllocGuard3D<int> dst_allocation(extent);
|
|
HIP_CHECK(hipSetDevice(src_device));
|
|
benchmark.Run(dst_allocation.pitched_ptr(), src_allocation.pitched_ptr(),
|
|
dst_allocation.extent(), kind, stream);
|
|
}
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Executes `hipMemcpy3DAsync` from Device to Host:
|
|
* -# Allocation size
|
|
* - Small: 4 KB x 16 B x 4 B
|
|
* - Medium: 4 MB x 16 B x 4 B
|
|
* - Large: 16 MB x 16 B x 4 B
|
|
* Test source
|
|
* ------------------------
|
|
* - performance/api/memcpy/hipMemcpy3DAsync.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 5.2
|
|
*/
|
|
HIP_TEST_CASE(Performance_hipMemcpy3DAsync_DeviceToHost) {
|
|
const auto width = GENERATE(4_KB, 4_MB, 16_MB);
|
|
RunBenchmark(make_hipExtent(width, 16, 4), hipMemcpyDeviceToHost);
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Executes `hipMemcpy3DAsync` from Host to Device:
|
|
* -# Allocation size
|
|
* - Small: 4 KB x 16 B x 4 B
|
|
* - Medium: 4 MB x 16 B x 4 B
|
|
* - Large: 16 MB x 16 B x 4 B
|
|
* Test source
|
|
* ------------------------
|
|
* - performance/api/memcpy/hipMemcpy3DAsync.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 5.2
|
|
*/
|
|
HIP_TEST_CASE(Performance_hipMemcpy3DAsync_HostToDevice) {
|
|
const auto width = GENERATE(4_KB, 4_MB, 16_MB);
|
|
RunBenchmark(make_hipExtent(width, 16, 4), hipMemcpyHostToDevice);
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Executes `hipMemcpy3DAsync` from Host to Host:
|
|
* -# Allocation size
|
|
* - Small: 4 KB x 16 B x 4 B
|
|
* - Medium: 4 MB x 16 B x 4 B
|
|
* - Large: 16 MB x 16 B x 4 B
|
|
* Test source
|
|
* ------------------------
|
|
* - performance/api/memcpy/hipMemcpy3DAsync.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 5.2
|
|
*/
|
|
HIP_TEST_CASE(Performance_hipMemcpy3DAsync_HostToHost) {
|
|
const auto width = GENERATE(4_KB, 4_MB, 16_MB);
|
|
RunBenchmark(make_hipExtent(width, 16, 4), hipMemcpyHostToHost);
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Executes `hipMemcpy3DAsync` from Device to Device with peer access disabled:
|
|
* -# Allocation size
|
|
* - Small: 4 KB x 16 B x 4 B
|
|
* - Medium: 4 MB x 16 B x 4 B
|
|
* - Large: 16 MB x 16 B x 4 B
|
|
* Test source
|
|
* ------------------------
|
|
* - performance/api/memcpy/hipMemcpy3DAsync.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 5.2
|
|
*/
|
|
HIP_TEST_CASE(Performance_hipMemcpy3DAsync_DeviceToDevice_DisablePeerAccess) {
|
|
const auto width = GENERATE(4_KB, 4_MB, 16_MB);
|
|
RunBenchmark(make_hipExtent(width, 16, 4), hipMemcpyDeviceToDevice);
|
|
}
|
|
|
|
/**
|
|
* Test Description
|
|
* ------------------------
|
|
* - Executes `hipMemcpy3DAsync` from Device to Device with peer access enabled:
|
|
* -# Allocation size
|
|
* - Small: 4 KB x 16 B x 4 B
|
|
* - Medium: 4 MB x 16 B x 4 B
|
|
* - Large: 16 MB x 16 B x 4 B
|
|
* Test source
|
|
* ------------------------
|
|
* - performance/api/memcpy/hipMemcpy3DAsync.cc
|
|
* Test requirements
|
|
* ------------------------
|
|
* - HIP_VERSION >= 5.2
|
|
*/
|
|
HIP_TEST_CASE(Performance_hipMemcpy3DAsync_DeviceToDevice_EnablePeerAccess) {
|
|
if (HipTest::getDeviceCount() < 2) {
|
|
HIP_SKIP_TEST(HipTest::SkipReason::kFewerThanTwoGpus);
|
|
}
|
|
const auto width = GENERATE(4_KB, 4_MB, 16_MB);
|
|
RunBenchmark(make_hipExtent(width, 16, 4), hipMemcpyDeviceToDevice, true);
|
|
}
|
|
|
|
/**
|
|
* End doxygen group memcpy.
|
|
* @}
|
|
*/
|